/*
 * Quake.  An Application for processing Seismic data from the USGS Earthquake feed
 * --------------------------------------------------------------------------------
 * Developed by Jim Blackhurst    [http://jimblackhurst.com/wiki/index.php/SReflec]
 * Uses RomeFeeder Library for Processing   [http://www.brunonadeau.com/romefeeder]
 *
 * This application will output data to a serial port using a packet format:
 * [001][mag*10] for use with the Seismic Reflector Hardware.  
 *
 */

import net.obxlabs.romefeeder.*; //romefeeder
import com.sun.syndication.feed.synd.*; //ROME 
import processing.serial.*;

import ddf.minim.*;

int swidth = 146; //set screen width
int sheight = 180; //set screen height
int init_run = 0; // Flag for preventing lots of noise and shaking while parseing the RSS feed on load 
int mute = 0; // is the audio alert muted?
int connected = 0; // is the hardware connected

Serial arduino;

AudioPlayer player;
Minim minim;
Feeder feeder;  // the feeder
Quake myQuake; // The Quake Class
int feedRate = 10; // rate for displaying posts (in miliseconds)
int feedLast = 0;  // time of the last displayed post
int num_entries = 0; // the number of entries in the feed at load time.
int count = 0; // A counter incrementing per feed processed
SyndEntry entry; // feed entry
PImage globeImage; // an Image object for the globe
PImage globeMask; // an Image object for the globe mask
PImage buttons; // an Image object for the buttons
PFont smallFont; // create a font 





void setup()
{
  //set up the window
  size(swidth,sheight, JAVA2D);
  //background(0);
  //initiate the classes
  feeder = new Feeder();
  myQuake = new Quake();
  //set up audio
  minim = new Minim(this);
  // Load a font into the font object
  smallFont = createFont("Verdana", 8);
  //load in the buttons
  buttons = loadImage("buttons.png");
  // Load the Alert sample into the player 
  player = minim.loadFile("carterattack.mp3", 2048);
 
 
 // Initialise the Serial connection
 // CHANGE THE "COM6" VALUE TO WHAT EVER COM PORT YOU ARE USING!
  arduino = new Serial(this, "COM6", 9600); 
  
  //Set the variables for the Feeder Object 
  feeder.verbose = false;
  feeder.sort(Feeder.UPDATED_DATE);

  //load the feed  
  feeder.load("http://earthquake.usgs.gov/eqcenter/catalogs/1day-M2.5.xml");
  //set the update interval to check for new posts in the loaded feed(s)
  feeder.setUpdateInterval(30*1000); // milliseconds
  //start updating
  feeder.startUpdate();
  // set flag to show that this is the initial pass through the RSS feed...  
  // ...We don't want the siren and hardware going bananas at this stage. 
  
  // increment the counter for good luck
  count++;
}






void draw()
{

  
  
  // Test to see if there is a new item in the feed
  if ((feeder.hasNext()) && (millis()-feedLast >= feedRate))
  {
    // clear the screen
    background(0);
    // get the total number of entries in the feed when it is first loaded, and only update it once (when count == 0)
    if (num_entries == 0 ) num_entries = feeder.count();
    println("number of entries:"+num_entries+" current count:"+count);

    //if this entry is new in the list (wasm't there at load, then sound the alarm!
    if (count>num_entries) alert(int(myQuake.mg*10));  
    
    //assign the next entry in the feed to an 'Entry' Object  
    entry = feeder.next();
    
    // Update the 'Quake' object with data from the current 'Entry'
    myQuake.update();
    // Request an image from the URL in the feed
    globeImage = requestImage(myQuake.iURL);
    //Load the Globe Mask, I do this becuase I can't figure out CSG.
    globeMask = loadImage("earth_mask.png");
    
    displayGUI();
    
    // Output Debug Text
    print("Magnitude:"+ myQuake.mg+". ");
    print("Location:"+ myQuake.loc+". ");
    //println("[URL:"+ myQuake.iURL+"]");
    println("");
    
    
    
    //Pause to give the image a moment to come down the wire 
    delay(400);
    
          //test to see if the Image has been retrived
          if (globeImage.width == 0)
          {
            // Image is not yet loaded
            println("Warning: Image not Loaded");
          } 
          else if (globeImage.width == -1) 
          {
            // This means an error occurred during image loading
            println("Warning: Error Loading Image");
          } 
          else
          {
            // Image is ready to go, draw it
            image(globeImage, 0, 0);
            image(globeMask, 0, 0);
            render_text();
          }
          
     
    // increment the counter
    count++;
       
          
    // reset the time      
    feedLast = millis();
  }
}

 	

void mouseMoved() 
{
  displayGUI();
}


void render_text()
{
  fill(255);
  textFont(smallFont);
  textAlign(LEFT);
  text("Latest:"+myQuake.time+"     Magnitude:"+myQuake.mg, 2, sheight-28);
  textAlign(CENTER);
  text(myQuake.loc, swidth/2, sheight-18);
  //text("Connected to COM xx", 2, sheight-2);
}

void alert(int mag)
{
  // scale the magnitude for the arduino
  // the motors won't sping below a input value of about 200, so we'll scale that to be our lowest alert level.
  int send_mag = int(map(mag, 0, 100, 200, 255));


  //triggered when a new quake is detected.
  println("");
  print("***ALERT!***");
  print(mag);
  print(" - sending:");
  print(send_mag);
  println("");
  


  
  //If connected, send Alert signal over to the arduino
  
  if (connected == 1)
  {
    //header byte
    arduino.write(1);
    //value to write
    arduino.write(int(send_mag));
  }   
  //sound the alarm!
  player.play();  
}



void mouseClicked()
{
 //mute the audio 
 if ((mouseX>0) && (mouseX<0+29) && (mouseY>165) && (mouseY<165+12) && (mute==0))
  { 
    mute=1;
    println("Audio muted");
    player.pause();
    player.mute();
    return;
  }
 //unmute the audio 
 if ((mouseX>0) && (mouseX<0+29) && (mouseY>165) && (mouseY<165+12) && (mute==1))
  { 
    mute=0;
    println("Audio On");
    player.unmute();
    return;
  }

 //Connect  
  if ((mouseX>29) && (mouseX<29+29) && (mouseY>165) && (mouseY<165+12) && (connected==0))
  { 
    connected=1;
    println("Conected");
    return;
  }
 //Disconnect  
  if ((mouseX>29) && (mouseX<29+29) && (mouseY>165) && (mouseY<165+12) && (connected==1))
  { 
    connected=0;
    println("DISconected");
    return;
  }

  //Exit
  if ((mouseX>116) && (mouseX<116+29) && (mouseY>165) && (mouseY<165+12))
  { 
    exit();
    return;
  }


  //OpenWebPage
  if ((mouseX>87) && (mouseX<87+29) && (mouseY>165) && (mouseY<165+12))
  {
    link("http://earthquake.usgs.gov"+myQuake.wURL, "_new"); 
    return;
  }
  
  
  //test alert
    if ((mouseX>58) && (mouseX<58+29) && (mouseY>165) && (mouseY<165+12))
   {
    alert(50);
   };
  



}

/*
int test_connect(void)
{
  //test to see if the hardware is connected.
  //send value 1 as a byte and see what we get back.

  //send a token to the hardware
  Serial.print(1);
  //wait an unfeesably long time for the hardware to respond... say 1/2 a second or so
  delay(500);
  // check the serial.in buffer.  It is is empty or contains something other 
  // than a 1 then assume we are not connected
  if (bytein==1) connected=1;
  else connected=0;
}
*/






void displayGUI()
{
  fill(0);
  rect(0, 166, 146, 14);
  
  // draw an active indicator on the first two buttons
  fill(40,10,60);
  if (mute==0)  rect(0, 165, 29, 14);  
  if (connected==1)  rect(29, 165, 29, 14);  
  // draw a mouse over highlight
  fill(73,0,73);
  if ((mouseX>0) && (mouseX<0+29) && (mouseY>165) && (mouseY<165+12))  rect(0, 165, 29, 14);
  if ((mouseX>29) && (mouseX<29+29) && (mouseY>165) && (mouseY<165+12))  rect(29, 165, 29, 14);
  if ((mouseX>58) && (mouseX<58+29) && (mouseY>165) && (mouseY<165+12)) rect(58, 165, 29, 14);
  if ((mouseX>87) && (mouseX<87+29) && (mouseY>165) && (mouseY<165+12)) rect(87, 165, 29, 14);
  if ((mouseX>116) && (mouseX<116+29) && (mouseY>165) && (mouseY<165+12)) rect(116, 165, 29, 14);
  image(buttons, 1, 166);
}





//Class 'definition' for Quake object 
class Quake 
{ 
  //Declare variables for Magnitude, Location and Image URL
  float mg;
  String loc;
  String iURL;
  String time;
  String wURL;
  

  //Define a method for updateing the Quake object data by parsing the RSS content
  void update()
  {
    //Update the Magnitude of the Quake
    String[] contents = splitTokens(entry.getTitle(), ","); 
    mg = float(contents[0].substring(2, 5));

    //Update the Location String
    loc = contents[1];
    
    //Update the Image URL
    SyndContent content = entry.getDescription();
    String[] icontent = splitTokens(content.getValue(),"\"");
    iURL = icontent[1];
    
    //Update Quake Time
    String[] time_s = splitTokens(icontent[8]," ");
    time = time_s[4]; 
    
    //update Quake webpage URL
    wURL = entry.getLink();
  } 
}


void stop()
{
  // always close Minim audio classes when you are done with them
  player.close();
  minim.stop();
  super.stop();
}
//END
